diff --git a/libc/inet/resolv.c b/libc/inet/resolv.c
index 92a65d0dc..7d952af28 100644
--- a/libc/inet/resolv.c
+++ b/libc/inet/resolv.c
@@ -255,6 +255,7 @@ Domain name in a message can be represented as either:
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <bits/uClibc_mutex.h>
+#include <fcntl.h>
 #include "internal/parse_config.h"
 
 /* poll() is not supported in kernel <= 2.0, therefore if __NR_poll is
@@ -1045,6 +1046,26 @@ static int __decode_answer(const unsigned char *message, /* packet */
 	return i + RRFIXEDSZ + a->rdlength;
 }
 
+#define DNSRAND_RESEED_CNT 128
+int dnsrand_next(int urand_fd, int def_value) {
+	static int cnt = -1;
+	if (urand_fd == -1) return def_value;
+	cnt += 1;
+	if ((cnt%DNSRAND_RESEED_CNT) == 0) {
+		int ival;
+		if(read(urand_fd, &ival, sizeof(int)) != sizeof(int)) return def_value;
+		srandom(ival);
+	}
+	int val = random(); // We dont need reproducible pseudo-random seq behaviour, so not bothering with random_r
+	return val;
+}
+
+int dnsrand_setup(int *urand_fd, int def_value) {
+	*urand_fd = open("/dev/urandom", O_RDONLY);
+	if (*urand_fd == -1) return def_value;
+	return dnsrand_next(*urand_fd, def_value);
+}
+
 /* On entry:
  *  a.buf(len) = auxiliary buffer for IP addresses after first one
  *  a.add_count = how many additional addresses are there already
@@ -1070,6 +1091,7 @@ int __dns_lookup(const char *name,
 	/* Protected by __resolv_lock: */
 	static int last_ns_num = 0;
 	static uint16_t last_id = 1;
+	static int urand_fd = -1;
 
 	int i, j, fd, rc;
 	int packet_len;
@@ -1149,7 +1171,7 @@ int __dns_lookup(const char *name,
 		}
 		/* first time? pick starting server etc */
 		if (local_ns_num < 0) {
-			local_id = last_id;
+			local_id = dnsrand_setup(&urand_fd, last_id);
 /*TODO: implement /etc/resolv.conf's "options rotate"
  (a.k.a. RES_ROTATE bit in _res.options)
 			local_ns_num = 0;
@@ -1159,7 +1181,7 @@ int __dns_lookup(const char *name,
 		}
 		if (local_ns_num >= __nameservers)
 			local_ns_num = 0;
-		local_id++;
+		local_id = dnsrand_next(urand_fd, local_id++);
 		local_id &= 0xffff;
 		/* write new values back while still under lock */
 		last_id = local_id;
